class Context:
    """ Queue wrappper for python user"""
    def __init__(self, channel, delivery_info):
        self.channel = channel
        self.delivery_info = delivery_info

    def success(self):
        self.channel.basic_ack(self.delivery_info.delivery_tag)

    def fail(self):
        self.channel.basic_nack(self.delivery_info.delivery_tag, requeue=True)

    def reject(self):
        self.channel.basic_nack(self.delivery_info.delivery_tag, requeue=False)



class Base:
    connection = None
    consumers = []

    def get_consumers(self):
        return self.consumers

    def get_connection(self):
        raise NotImplementedError('Must be implemented') #pragma: no cover

    @classmethod
    def consume(cls, event_name, callback,final_method_name=None, continuous=True):
        if(cls.consumers[event_name]):
            return cls.consumers[event_name]

        cls.consumers[event_name] = cls().consume(event_name, callback)

    @classmethod
    def produce(cls, events):
        cls().produce(events)



import pytest
import os

BASE_DIR: str = 'event_people'

@pytest.fixture(scope='function')
def setup():
    os.environ['RABBIT_URL'] = 'amqp://guest:guest@localhost:5672'
    os.environ['RABBIT_EVENT_PEOPLE_APP_NAME'] = 'service_name'
    os.environ['RABBIT_EVENT_PEOPLE_VHOST'] = 'event_people'
    os.environ['RABBIT_EVENT_PEOPLE_TOPIC_NAME'] = 'event_people'

    yield {'basedir': BASE_DIR}

    os.unsetenv('RABBIT_URL')
    os.unsetenv('RABBIT_EVENT_PEOPLE_APP_NAME')
    os.unsetenv('RABBIT_EVENT_PEOPLE_VHOST')
    os.unsetenv('RABBIT_EVENT_PEOPLE_TOPIC_NAME')

@pytest.fixture(scope='function')
def load_listener_test(setup):
    from event_people import BaseListener

    class TestListener(BaseListener):

        def __init__(self, context):
            self.context = context

        def pay(self, event):
            print(f"Paid {event.body['amount']} for {event.body['name']} ~> {event.name}")


        def receive(self, event):
            if event.body['amount'] > 500:
                print(f"Received {event.body['amount']} from {event.body['name']} ~> {event.name}")
            else:
                print('[consumer] Got SKIPPED message')

    return TestListener, setup




import os

from event_people.broker.rabbit_broker import RabbitBroker

class Config:
    """ Class that load all enviroment variable necessary to feature works"""

    APP_NAME = os.environ['RABBIT_EVENT_PEOPLE_APP_NAME']
    TOPIC_NAME = os.environ['RABBIT_EVENT_PEOPLE_TOPIC_NAME']
    VHOST = os.environ['RABBIT_EVENT_PEOPLE_VHOST']
    RABBIT_URL = os.environ['RABBIT_URL']
    broker = None

    @classmethod
    def get_broker(cls):
        cls.broker = cls.broker or RabbitBroker()

        return cls.broker



import pika
import pytest
from mock import patch


class TestTopic:

    def test_get_topic_with_channel(self, setup):
        from event_people import RabbitBroker
        from event_people import Topic

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            Topic.get_topic(channel=channel)

    def test_get_topic_whithout_channel(self, setup):
        from event_people import Topic

        with pytest.raises(ValueError):
            Topic.get_topic(channel=None)

    def test_produce_topic_with_channel(self, setup):
        from event_people import RabbitBroker
        from event_people import Event
        from event_people import Topic

        body = { 'amount': 350, 'name': 'George' }
        event = Event(name='resource.custom.pay', body=body)
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            Topic.produce(channel=channel, event=event)

    def test_produce_topic_without_channel(self, setup):
        from event_people import Event
        from event_people import Topic
        with pytest.raises(ValueError):
            body = { 'amount': 350, 'name': 'George' }
            event = Event(name='resource.custom.pay', body=body)
            Topic.produce(channel=None, event=event)



class TestConfig:


    def test_env_exists(self, setup):
        from event_people.config import Config
        assert Config().RABBIT_URL == 'amqp://guest:guest@localhost:5672'
        assert Config().APP_NAME == 'service_name'
        assert Config().VHOST == 'event_people'
        assert Config().TOPIC_NAME == 'event_people'

    def test_config_get_broker(self, setup):
        from event_people.config import Config
        config = Config()
        broker = config.get_broker()
        assert broker is not None






from event_people.config import Config

class Listener:
    @staticmethod
    def on(event_name, callback = None):
        broker_callback = callback if callback else Listener.basic_callback
        queue_names = [event_name]
        broker = Config.get_broker()
        channel = broker.get_connection()

        if len(event_name.split('.')) == 3:
            queue_names = [f'{event_name}.all', f'{event_name}.{Config.APP_NAME}']

        for queue_name in queue_names:
            broker.consume(queue_name, broker_callback, False)

        try:
            channel.start_consuming()
        except KeyboardInterrupt:#pragma: no cover
            channel.stop_consuming()#pragma: no cover

    def basic_callback(event, context):
        print(event.name)#pragma: no cover
        print(event.header)#pragma: no cover
        print(event.body)#pragma: no cover
        context.success()#pragma: no cover




from event_people.config import Config

class Emitter:
    @classmethod
    def trigger(cls, *events):
        cls().itrigger(events)

    def itrigger(self, events):
        broker = Config.get_broker()
        channel = broker.get_connection()

        broker.produce(events)

        try:
            channel.start_consuming()
        finally:
            channel.stop_consuming()

        return events




import pika
from mock import patch

class TestEmitter:

    def test_with_one_event(self, setup):
        from event_people import Emitter
        from event_people import Event
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            body = { 'amount': 350, 'name': 'George' }
            event = Event(name='resource.custom.pay', body=body)
            Emitter.trigger(event)

    def test_with_more_than_one_event(self, setup):
        from event_people import Emitter
        from event_people import Event
        events = []
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            body = { 'amount': 350, 'name': 'George' }
            event1 = Event(name='resource.custom.pay', body=body)
            events.append(event1)

            body = { 'amount': 700, 'name': 'Barnie' }
            event2 = Event(name='resource.custom.pay.all', body=body)
            events.append(event2)
            Emitter.trigger(events)



from mock import patch
import pika

class TestListener:

    def callback(event, context):
        print(event.name)
        print(event.header)
        print(event.body)
        context.success()


    def test_listen_event_with(self, setup):
        from event_people import Listener
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            Listener.on('resource.custom.receive.all', TestListener.callback)

    def test_listen_passing_callback_no_callback(self, setup):
        from event_people import Listener

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            Listener.on('resource.custom.receive.all', TestListener.callback)

    def test_listen_event_name_with_three_parts(self, setup):
        from event_people import Listener

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            Listener.on('resource.custom.receive', TestListener.callback)



from event_people.config import Config

class ListenerManager:
    _listeners = []

    @classmethod
    def add_listener(cls, listener_class, callback, event_name):
        configuration = ListenerConfiguration(listener_class, callback, event_name)
        cls._listeners.append(configuration)

    @classmethod
    def bind_all_listeners(cls):
        broker = Config.get_broker()

        for listener in cls._listeners:
            broker.consume(listener.event_name, listener.listener_class.callback, final_method_name=listener.callback)


class ListenerConfiguration:
    def __init__(self, listener_class, callback, event_name):
        self.listener_class = listener_class
        self.callback = callback
        self.event_name = event_name



import mock
from pika.spec import Basic

class TestContext:

    @mock.patch('pika.BlockingConnection')
    def test_context_sucess_message(self, connection, setup):
        from event_people import Context
        delivery_tag = Basic.Deliver('consumer_tag_')
        context = Context(connection, delivery_tag)
        context.success()

    @mock.patch('pika.BlockingConnection')
    def test_context_reject_message(self, connection, setup):
        from event_people.broker.rabbit.context import Context
        delivery_tag = Basic.Deliver('consumer_tag_')
        context = Context(connection, delivery_tag)
        context.reject()

    @mock.patch('pika.BlockingConnection')
    def test_context_fail_message(self, connection, setup):
        from event_people.broker.rabbit.context import Context
        delivery_tag = Basic.Deliver('consumer_tag_')
        context = Context(connection, delivery_tag)
        context.fail()



from event_people.broker.rabbit.queue import Queue
from event_people.broker.rabbit_broker import RabbitBroker
from event_people.broker.rabbit.context import Context
from event_people.event import Event
from event_people.listeners.base_listener import BaseListener
from event_people.listeners.listener_manager import ListenerManager
from event_people.emitter import Emitter
from event_people.daemon import Daemon
from event_people.listener import Listener
from event_people.config import Config
from event_people.broker.rabbit.topic import Topic


import pytest
from mock import patch
import pika

from pika.spec import Basic


class TestQueue:

    def callback(event, context, final_method):
        print(event.name)
        print(event.header)
        print(event.body)
        print(final_method)
        context.success()


    def test_create_queue_without_channel(self, setup):
        from event_people import Queue
        with pytest.raises(ValueError):
            q = Queue(None)
            assert q

    def test_create_queue_with_channel_ok(self, setup):
        from event_people import RabbitBroker
        from event_people import Queue
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            q = Queue(channel)
            assert q


    def test_subscribe_queue_ok(self, setup):
        from event_people import RabbitBroker
        from event_people import Queue
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            Queue.subscribe(channel, 'resource.custom.receive.all', True, TestQueue.callback)


    def test_subscribe_queue_name_less_four_parts(self, setup):
        from event_people import RabbitBroker
        from event_people import Queue
        with pytest.raises(ValueError):
            with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
                rabbit = RabbitBroker()
                channel = rabbit.get_connection()
                Queue.subscribe(channel, 'resource.custom', False, TestQueue.callback)

    def test_subscribe_with_fourth_queue_name_different_than_all(self, setup):
        from event_people import RabbitBroker
        from event_people import Queue
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            Queue.subscribe(channel, 'resource.custom.recieve.action', False, TestQueue.callback)

    def test_callback_subscribe_sucessffuly(self, setup):
        from event_people import RabbitBroker
        from event_people import Context
        from event_people import Queue

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            delivery_tag = Basic.Deliver('consumer_tag_')
            delivery_tag.routing_key = 'resource.custom.pay'
            body = { 'amount': 350, 'name': 'George' }
            context = Context(channel, delivery_tag)
            Queue._callback(None, channel=channel, delivery_info= delivery_tag, properties=context, payload=body, args=[False, TestQueue.callback, 'callback'])



import os
from functools import partial
from event_people.event import Event
from .context import Context

class Queue:
    TOPIC_NAME = os.environ['RABBIT_EVENT_PEOPLE_TOPIC_NAME']
    APP_NAME = os.environ['RABBIT_EVENT_PEOPLE_APP_NAME']

    """ Queue wrappper for python user"""
    def __init__(self, channel):
        if channel is None:
            raise ValueError("Channel must be defined.")

        channel.basic_qos(prefetch_count=1)
        self._channel = channel

    @classmethod
    def subscribe(cls, channel, event_name, continuous, callback, final_method_name=None):
        cls(channel).isubscribe(event_name, continuous, callback, final_method_name)

    def isubscribe(self, event_name, continuous, callback, final_method_name=None):
        queue_name = self._define_queue(event_name)

        on_message_callback = partial(self._callback, args=(continuous, callback, final_method_name))
        self._channel.basic_consume(
            queue=queue_name, on_message_callback=on_message_callback, auto_ack=False)

    def _define_queue(self, event_name):
        routing_key = '.'.join(event_name.split('.')[0:4])

        queue_name = self._queue_name(routing_key)
        self._channel.queue_declare(
            queue_name, durable=True, exclusive=False)

        print(routing_key)

        self._channel.queue_bind(
            exchange=self.TOPIC_NAME, queue=queue_name, routing_key=routing_key)

        return queue_name

    def _callback(self, channel, delivery_info, properties, payload, args):
        continuous, callback, final_method_name = args
        event_name = delivery_info.routing_key

        event = Event(event_name, payload)
        context = Context(channel, delivery_info)

        callback(event, context, final_method_name)

        if not continuous:
            channel.stop_consuming()

    def _queue_name(self, routing_key) -> str:
        broken_event_name = routing_key.split('.')
        if len(broken_event_name) < 3:
            raise ValueError("queue name must follow the pattern, resource.origin.action or resource.origin.action.destination")

        event_name = '.'.join(broken_event_name[0:4])

        if broken_event_name[3] != 'all':
            event_name = f"{'.'.join(broken_event_name[0:3])}.all"

        return f'{self.APP_NAME.lower()}-{event_name.lower()}'



import os
import json

class Topic:
    """ Queue wrappper for python user"""
    TOPIC_NAME = os.environ['RABBIT_EVENT_PEOPLE_TOPIC_NAME']
    EXCHANGE_TYPE = 'topic'

    def __init__(self, channel):
        if channel is None:
            raise ValueError("Channel must be defined.")

        self._channel = channel

    @classmethod
    def get_topic(cls, channel):
        cls(channel).iget_topic()

    def iget_topic(self):
        self._channel.exchange_declare(
            self.TOPIC_NAME, exchange_type=self.EXCHANGE_TYPE, passive=True, durable=True)

        return self._channel

    @classmethod
    def produce(cls, channel, event):
        cls(channel).iproduce(event)

    def iproduce(self, event):
        topic = self.iget_topic()
        body = json.dumps({'body': event.body, 'headers': event.header.__dict__}, indent=2).encode('utf-8')

        topic.basic_publish(exchange=self.TOPIC_NAME, routing_key=event.name, body=body)



from mock import patch
import pika

class TestDeamon:

    def test_start_deamon(self, setup):
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            from event_people import Daemon
            Daemon.start()




import ast
import pytest
import json

class TestEvent:

    def test_load_event_type_is_dict(self, setup):
        from event_people import Event

        event_name = 'resource.custom.receive'
        body = b"{'amount': 35, 'name': 'Peter'}"

        event = Event(name=event_name, body=body)

        assert event.name == f'{event_name}.all'
        assert ast.literal_eval(body.decode('utf-8')) == event.body

    def test_load_event_with_invalid_name(self, setup):
        from event_people import Event

        event_name = 'resource.custom'
        body = b"{'amount': 35, 'name': 'Peter' }"

        with pytest.raises(ValueError):
            Event(name=event_name, body=body)

    def test_get_payload_succeffuly(self, setup):
        from event_people import Event
        event_name = 'resource.custom.receive'
        body = b"{'amount': 35, 'name': 'Peter'}"

        event = Event(name=event_name, body=body)

        resp = json.loads(event.payload())

        header = resp['headers']
        assert header['app'] == 'service_name'
        assert header['resource'] == 'resource'
        assert header['origin'] == 'custom'

        assert resp['body'] == ast.literal_eval(body.decode('utf-8'))

    def test_get_payload_with_empty_body(self, setup):
        from event_people import Event
        event_name = 'resource.custom.receive'
        body = {}

        event = Event(name=event_name, body=body)

        resp = json.loads(event.payload())

        assert resp['body'] == body


import pika
from mock import patch

class TestListenerManager:

    def test_add_with_one_listener(self, load_listener_test):
        from event_people import ListenerManager
        ListenerManager.add_listener(listener_class=load_listener_test, callback=None, event_name='resource.custom.pay')
        assert len(ListenerManager._listeners) == 1


    def test_add_listener_with_more(self, load_listener_test):
        from event_people import ListenerManager
        ListenerManager._listeners.clear()
        ListenerManager.add_listener(listener_class=load_listener_test, callback=None, event_name='resource.custom.pay')
        ListenerManager.add_listener(listener_class=load_listener_test, callback=None, event_name='resource.custom.service.all')
        assert len(ListenerManager._listeners) == 2

    def test_bind_all_listeners(self, load_listener_test):
        from event_people import ListenerManager
        ListenerManager._listeners.clear()
        listener_test, setup = load_listener_test
        ListenerManager.add_listener(listener_class=listener_test, callback=None, event_name='resource.custom.pay.service')
        ListenerManager.add_listener(listener_class=listener_test, callback=None, event_name='resource.custom.service.all')
        assert len(ListenerManager._listeners) == 2

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            ListenerManager.bind_all_listeners()


from event_people.config import Config
from event_people.listeners.listener_manager import ListenerManager

class Daemon:
    @classmethod
    def start(cls):
        channel = Config.get_broker().get_connection()

        ListenerManager.bind_all_listeners()

        try:
            channel.start_consuming()
        except KeyboardInterrupt: #pragma: no cover
            channel.stop_consuming()##pragma: no cover



from mock import patch
import pika

class TestRabbitBroker:

    def callback(event, context):
        print(event.name)
        print(event.header)
        print(event.body)
        context.success()

    def test_get_connection_ok(self, setup):
        from event_people import RabbitBroker

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            channel = rabbit.get_connection()
            assert channel

    def test_get_connection_with_open_status(self,setup):
        from event_people import RabbitBroker

        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection) as mocked_connection:
            mocked_connection.is_closed = False
            rabbit = RabbitBroker()
            rabbit.connection = mocked_connection
            channel = rabbit.get_connection()
            assert channel

    def test_consume_event(self, setup):
        from event_people import RabbitBroker
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            rabbit.consume('resource.custom.recieve.action', TestRabbitBroker.callback, False)

    def test_produce_list_events(self, setup):
        from event_people import RabbitBroker
        from event_people import Event
        events = []
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()
            event_name = 'resource.custom.receive'
            body = { 'amount': 350, 'name': 'George' }

            events.append(Event(event_name, body))

            event_name = 'resource.custom.receive'
            body = { 'amount': 550, 'name': 'James' }

            events.append(Event(event_name, body))

            event_name = 'resource.custom.private.service'
            body = { 'message': 'Secret' }

            events.append(Event(event_name, body))

            event_name = 'resource.origin.action'
            body = { 'bo': 'dy' }
            schema_version = 4.2

            events.append(Event(event_name, body, schema_version))

            rabbit.produce(events)

    def test_produce_unique_event(self, setup):
        from event_people import RabbitBroker
        from event_people import Event
        with patch('{0}.broker.rabbit_broker.pika.BlockingConnection'.format(setup['basedir']), spec=pika.BlockingConnection):
            rabbit = RabbitBroker()

            event_name = 'resource.origin.action'
            body = { 'bo': 'dy' }
            schema_version = 4.2

            event = Event(event_name, body, schema_version)

            rabbit.produce(event)


from event_people.config import Config
from event_people.listeners.listener_manager import ListenerManager

class BaseListener:
    def __init__(self, context):
        self._context = context

    @classmethod
    def bind_event(cls, event_name, callback):
        app_name = Config.APP_NAME
        if len(event_name.split('.')) <= 3:
            ListenerManager.add_listener(
                listener_class=cls,
                callback=callback,
                event_name=cls.fixed_event_name(event_name, 'all')
            )
            ListenerManager.add_listener(
                listener_class=cls,
                callback=callback,
                event_name=cls.fixed_event_name(event_name, app_name)
            )
        else:
            ListenerManager.add_listener(
                listener_class=cls,
                callback=callback,
                event_name=cls.fixed_event_name(event_name, app_name)
            )

    @classmethod
    def callback(cls, event, context, final_method_name):
        instance = cls(context)
        method = getattr(instance, final_method_name)

        method(event)

    @classmethod
    def fixed_event_name(cls, event_name, postfix):
        routing_key = event_name
        splitted = event_name.split('.')

        if len(splitted) <= 3:
            routing_key = f'{event_name}.{postfix}'
        elif len(splitted) == 4:
            base_name = '.'.join(splitted[0:3])
            routing_key = f'{base_name}.{postfix}'

        return routing_key

    def success(self):
        self._context.success()

    def fail(self):
        self._context.fail()

    def reject(self):
        self._context.reject()



import os
import pika
from pika.exceptions import AMQPConnectionError, StreamLostError

from event_people.broker.rabbit.queue import Queue
from event_people.broker.base import Base
from event_people.broker.rabbit.topic import Topic

class RabbitBroker(Base):
    VHOST = os.environ['RABBIT_EVENT_PEOPLE_VHOST']
    RABBIT_URL = os.environ['RABBIT_URL']

    def get_connection(self):
        try:
            if self.connection and not self.connection.is_closed:
                return self.connection.channel()

            return self._channel()
        except (AMQPConnectionError, StreamLostError) as error:
            return self._channel()

    def consume(self, event_name, callback, final_method_name=None, continuous=True):
        Queue.subscribe(self.get_connection(), event_name, continuous, callback, final_method_name)

    def produce(self, events):
        events = events if hasattr(events, "__len__") else [events]

        for event in events:
            if hasattr(event, "__len__"):
                for item in event:
                    Topic.produce(self.get_connection(), item)
            else:
                Topic.produce(self.get_connection(), event)


    def _channel(self):
        self.connection = pika.BlockingConnection(self._parameters())

        return self.connection.channel()

    def _parameters(self):
        return pika.connection.URLParameters(self._full_url())

    def _full_url(self):
        return f'{self.RABBIT_URL}/{self.VHOST}'



"""
    Wrap all logic of an event, whenever you receive or want to send

"""
import json
import os
import ast

class Header:

    def __init__(self, app, resource, origin, action, destination, schema_version=None):
        self.app = app
        self.resource = resource
        self.origin = origin
        self.action = action
        self.destination = destination
        self.schema_version = schema_version or 1.0

    def __str__(self):
        return f'{self.app}.{self.resource}.{self.origin}.{self.action}.{self.destination}.{self.schema_version}'


class Event(object):
    """
        This class represents the pattern routing key got message_
    """
    APP_NAME = os.environ['RABBIT_EVENT_PEOPLE_APP_NAME']

    def __init__(self, name, body, schema_version = 1.0):
        self.name = self.__fix_name__(name)
        self.__generate_header__(schema_version)
        body_dict = body

        if not isinstance(body, dict):
            body_dict = ast.literal_eval(body.decode('utf-8'))

        self.header.schema_version = schema_version

        self.body = body_dict if 'body' not in body_dict else body_dict['body']


    def __generate_header__(self, schema_version):
        resource, origin, action, destination = self.name.split(".")
        self.header = Header(app=self.APP_NAME ,resource=resource, origin=origin, action=action, destination=destination, schema_version=schema_version)

    def __fix_name__(self, name):
        if len(name.split('.')) < 3:
            raise ValueError("pattern argument error in event's name")

        return f'{name}.all' if len(name.split('.')) == 3 else name

    def payload(self):
       return json.dumps({"headers": self.header.__dict__, "body": self.body})
